% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wdpa_fetch.R
\name{wdpa_fetch}
\alias{wdpa_fetch}
\title{Fetch data from the World Database on Protected Areas}
\usage{
wdpa_fetch(
  x,
  wait = FALSE,
  download_dir = tempdir(),
  force_download = FALSE,
  n = NULL,
  verbose = interactive()
)
}
\arguments{
\item{x}{\code{character} country for which to download data. This argument
can be the name of the country (e.g. \code{"Liechtenstein"}) or the
ISO-3 code for the country (e.g. \code{"LIE"}). This argument can also
be set to \code{"global"} to download all of the protected areas available
in the database (approximately 1.1 GB).}

\item{wait}{\code{logical} if data is not immediately available for download
should the session be paused until it is ready for download? If argument
to \code{wait} is \code{FALSE} and the data is not ready then \code{NA}
will be returned. Defaults to \code{FALSE}.}

\item{download_dir}{\code{character} folder path to download the data.
Defaults to a temporary directory. To avoid downloading the
same dataset multiple times, it is recommended to use a persistent
directory (e.g. \code{rappdirs::user_data_dir("wdpar")}; see Examples below).}

\item{force_download}{\code{logical} if the data has previously been
downloaded and is available at argument to \code{download_dir}, should a
fresh copy be downloaded? Defaults to \code{FALSE}.}

\item{n}{\code{integer} number of records to import per data source.
Defaults to \code{NULL} such that all data are imported.}

\item{verbose}{\code{logical} should a progress on downloading data be
reported? Defaults to \code{TRUE} in an interactive session, otherwise
\code{FALSE}.}
}
\value{
\code{\link[sf:sf]{sf::sf()}} object.
}
\description{
Download data from the World Database on Protected Areas (WDPA)
(available at \url{https://www.protectedplanet.net/en}) and import it.
Note that data are downloaded assuming non-commercial use.
}
\details{
This function will download the specified protected area
data and return it. \strong{It is strongly recommended that the data be
cleaned prior to analysis}. Check out the
\code{\link[=wdpa_clean]{wdpa_clean()}} function to clean the data according to standard
practices. For information on this database,
prefer refer to the official manual
(\url{https://www.protectedplanet.net/en/resources/wdpa-manual}).

Please note that this function will sometimes return the error
\verb{PhantomJS signals port = 4567 is already in use}. This can occur
when you have previously run the function and terminated it early.
To address this issue, you will need to restart your computer.
}
\examples{
\dontrun{
# fetch data for Liechtenstein
lie_raw_data <- wdpa_fetch("Liechtenstein", wait = TRUE)

# print data
print(lie_raw_data)

# plot data
plot(lie_raw_data)

# fetch data for Liechtenstein using the ISO3 code
lie_raw_data <- wdpa_fetch("LIE", wait = TRUE)

# since data are saved in a temporary directory by default,
# a persistent directory can be specified to avoid having to download the
# same dataset every time the R session is restarted
lie_raw_data <- wdpa_fetch("LIE", wait = TRUE,
                           download_dir = rappdirs::user_data_dir("wdpar"))

# data for multiple countries can be downloaded separately and combined,
# this is useful to avoid having to download the global dataset
## load packages to easily merge datasets
library(dplyr)
library(tibble)

## define country names to download
country_codes <- c("LIE", "MHL")

## download data for each country
mult_data <- lapply(country_codes, wdpa_fetch, wait = TRUE)

## merge datasets together
mult_dat <- st_as_sf(as_tibble(bind_rows(mult_data)))

## print data
print(mult_dat)
}
}
\seealso{
\code{\link[=wdpa_clean]{wdpa_clean()}}, \code{\link[=wdpa_read]{wdpa_read()}},
\code{\link[=wdpa_url]{wdpa_url()}}, \code{\link[countrycode:countrycode]{countrycode::countrycode()}},
\url{https://www.protectedplanet.net/en},
\url{https://www.protectedplanet.net/en/resources/wdpa-manual}.
}
